#include "aircontrolpoint.h"
#include "ui_aircontrolpoint.h"
#include <QTimer>
using namespace BrisaUpnp;

AirControlPoint::AirControlPoint(QWidget *parent) :
   QMainWindow(parent),
   ui(new Ui::AirControlPoint),
   airControlPoint(new BrisaControlPoint)
{
   this->ui->setupUi(this);
   connect(this->airControlPoint,
      SIGNAL(deviceFound(BrisaControlPointDevice*)),
      this,
      SLOT(onNewDevice(BrisaControlPointDevice*)));
   connect(this->airControlPoint,
      SIGNAL(deviceGone(QString)),
      this,
      SLOT(onRemovedDevice(QString)));
   this->airControlPoint->start();
   this->airControlPoint->discover();
   this->setTime = false;
}

AirControlPoint::~AirControlPoint()
{
   delete this->ui;
   delete this->airControlPoint;
}

void AirControlPoint::restart()
{
   this->airControlPoint->stop();
   this->airControlPoint->start();
   this->airControlPoint->discover();
}

void AirControlPoint::onNewDevice(BrisaControlPointDevice *device)
{
   if (device->getAttribute(BrisaControlPointDevice::DeviceType).compare(DEVICE_TYPE))
          return;

   for (int i = 0; i < this->airDevices.size(); i++) {
      QString dev_udn = this->airDevices[i]->getAttribute(BrisaControlPointDevice::Udn);
      QString cur_dev_udn = device->getAttribute(BrisaControlPointDevice::Udn);
          if (dev_udn == cur_dev_udn) {
              airDevices[i] = c;
              return;
          }
   }
   this->airDevices.append(c);
   this->addAirDevice(c);
}

void AirControlPoint::onRemovedDevice(QString udn){
   this->airDevices.removeOne(getDeviceByUDN(udn));
   this->restart();
}

void AirControlPoint::addAirDevice(BrisaControlPointDevice* device){
   QString friendlyName = device->getAttribute(BrisaControlPointDevice::FriendlyName);
   QString udn = device->getAttribute(BrisaControlPointDevice::Udn);
   QString item = friendlyName + " - " + udn;
   QListWidgetItem* device = new QListWidgetItem();
   device->setData(Qt::UserRole, QString(udn));
   device->setText(item);
   this->ui->li_devices->addItem(device);
}

QString AirControlPoint::removeFromList(QString udn)
{
   int i = 0;
   for (; i < ui->li_devices->count(); i++) {
      QListWidgetItem* currentItem = ui->li_devices->item(i);
      QString cur_udn = currentItem->data(Qt::UserRole).toString();
      if (cur_udn == udn.split("::")[0])
         return ui->li_devices->takeItem(i)->text();
   }
}

BrisaControlPointDevice* AirControlPoint::getDeviceByUDN(QString udn)
{
   for (int i = 0; i < airDevices.size(); i++)
          if (udn.compare(airDevices[i]->getAttribute(BrisaControlPointDevice::udn)) == 0)
              return airDevices[i];
   return NULL;
}

void AirControlPoint::changeTemperature(BrisaControlPointDevice *device, QString temperature)
{
   BrisaControlPointService *service = device->getServiceByType(SERVICE_TYPE);
   BrisaInArgument inArguments;
   inArguments["NewTemperature"] = temperature;
   service->call("SetTemperature", inArguments);
}

void AirControlPoint::changeStatus(BrisaControlPointDevice *device, bool status)
{
   BrisaControlPointService *service = device->getServiceByType(SERVICE_TYPE);
   QMap<QString,QString> inArguments;
   inArguments["NewStatus"] = QString((status ? "1" : "0"));
   service->call("SetStatus", inArguments);
}

void AirControlPoint::on_bt_change_status_clicked()
{
   if (this->ui->li_devices->selectedItems().count() == 1) {
          QString deviceSelected = this->ui->li_devices->currentItem()->data(Qt::UserRole).toString();
          if (!deviceSelected.isEmpty()) {
              BrisaControlPointDevice *device = this->getDeviceByUDN(deviceSelected);
              this->changeStatus(device, this->on);
              if (this->ui->timeEdit->time() == QTime(0,0,0,0))
                     this->setTime = false;
              if (this->ui->timeEdit->time() != QTime(0,0,0,0) && !this->setTime) {
                     setTime = true;
            QTimer::singleShot((this->ui->timeEdit->time().hour() * 360000) +
                     (this->ui->timeEdit->time().minute()*60000) +
                     (this->ui->timeEdit->time().second()*1000),
                     this, SLOT(this->on_bt_change_status_clicked()));
                     this->decrementTime();
              }
      }
   }

   if (this->on) {
      this->ui->bt_change_status->setIcon(QIcon(":/figs/red.jpg"));
      this->on = false;
   } else {
          this->ui->bt_change_status->setIcon(QIcon(":/figs/green.jpg"));
          this->on = true;
   }
}

void AirControlPoint::on_bt_change_temperature_clicked()
{
   if (this->ui->li_devices->selectedItems().count() == 1) {
      QString temperature = this->ui->txt_temperature->text();
      QString deviceSelected = this->ui->li_devices->currentItem()->data(Qt::UserRole).toString();
      if (!temperature.isEmpty() && !deviceSelected.isEmpty()) {
              BrisaControlPointDevice *device = this->getDeviceByUDN(deviceSelected);
              this->changeTemperature(device, temperature);
          }
   }
}

void AirControlPoint::on_li_devices_clicked(QModelIndex index)
{
   if (this->ui->li_devices->selectedItems().count() == 1) {
      QString deviceSelected = this->ui->li_devices->currentItem()->data(Qt::UserRole).toString();
      if (!deviceSelected.isEmpty()) {
         BrisaControlPointDevice *device = this->getDeviceByUDN(deviceSelected);
              BrisaControlPointService *service = device->getServiceByType(SERVICE_TYPE);
              BrisaInArgument status;
         connect(service, SIGNAL(requestFinished(BrisaOutArgument, QString)),
            this, SLOT(this->receiveDeviceResponse(BrisaOutArgument, QString)));
              service->call("GetTemperature", status);
          }
   }
}

void AirControlPoint::receiveDeviceResponse(QString response, QString method)
{
   if (method == "GetTemperature") {
           this->ui->txt_temperature->setText(response["CurrentTemperature"]);
    }

    if (method == "GetStatus") {
      if (response["CurrentStatus"] == "0") {
                ui->pushButton->setIcon(QIcon(":/figs/red.jpg"));
               this->on = false;
            } else {
                ui->pushButton->setIcon(QIcon(":/figs/green.jpg"));
                this->on = true;
            }
     }
}

void AirControlPoint::on_bt_get_status_clicked()
{
   if (ui->li_devices->selectedItems().count() == 1) {
      QString deviceSelected = this->ui->li_devices->currentItem()->data(Qt::UserRole).toString();
      if (!deviceSelected.isEmpty()) {
         BrisaControlPointDevice *device = this->getDeviceByUDN(deviceSelected);
              BrisaControlPointService *service = device->getServiceByType(SERVICE_TYPE);
         BrisaInArgument status;
              connect(service, SIGNAL(requestFinished(BrisaOutArgument, QString)),
            this, SLOT(this->receiveDeviceResponse(BrisaOutArgument, QString)));
              service->call("GetStatus", status);
          }
   }
}

void AirControlPoint::decrementTime()
{
   if (this->ui->timeEdit->time() != QTime(0, 0, 0, 0)) {
      this->ui->timeEdit->setTime(this->ui->timeEdit->time().addSecs(-1));
      QTimer::singleShot(1000, this, SLOT(this->decrementTime()));     
   }
}

void AirControlPoint::changeEvent(QEvent *e)
{
   QMainWindow::changeEvent(e);
   switch (e->type()) {
      case QEvent::LanguageChange:
             ui->retranslateUi(this);
             break;
      default:
             break;
   }
}

@KT:Listing 9: main.cpp
@LI:
#include <QtGui/QApplication>
#include "aircontrolpoint.h"

int main(int argc, char *argv[]) {
   QApplication a(argc, argv);
   AirControlPoint cpWindow;
   cpWindow.show();
   return a.exec();
}
